package pgbyteabug;

import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.sql.Array;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import org.apache.commons.codec.binary.Hex;

public class Main {

    public static void main(String... args) throws Exception {
        MessageDigest hasher = MessageDigest.getInstance("SHA-256");

        Connection con = null;
        PreparedStatement pst = null;
        ResultSet rs = null;
        try {

            con = DriverManager.getConnection("jdbc:postgresql://localhost:5432/test", "postgres", "");
            con.setAutoCommit(false);

            pst = con.prepareStatement("drop table if exists byteatest");
            pst.execute();
            pst.close();

            // create table 
            pst = con.prepareStatement("create table byteatest (hash bytea)");
            pst.execute();
            pst.close();

            // generate values for inserting
            final List<byte[]> hashes = new ArrayList<>(3);
            hashes.add(hasher.digest("one".getBytes("UTF-8")));
            hashes.add(hasher.digest("two".getBytes("UTF-8")));
            hashes.add(hasher.digest("three".getBytes("UTF-8")));

            // insert values
            pst = con.prepareStatement("insert into byteatest (hash) values(?)");
            for (byte[] hash : hashes) {
                pst.setBytes(1, hash);
                pst.execute();
            }
            pst.close();

            // check if all rows were inserted
            pst = con.prepareStatement("select * from byteatest");
            rs = pst.executeQuery();
            while (rs.next()) {
                System.err.println(rs.getString(1));
            }
            rs.close();

            // select the table with WHERE hash = any(array-of-byte[])
            Array hashArray = con.createArrayOf("bytea", hashes.toArray());
            System.err.println(String.format("String representation of array: %s", hashArray.toString()));

            // this yields 0 rows 
            pst = con.prepareStatement("select count(*) from byteatest where hash = any(?)");
            pst.setArray(1, hashArray);
            rs = pst.executeQuery();
            rs.next();
            System.err.println(String.format("found %s rows with array of byte[]", rs.getInt(1)));
            rs.close();

            // select the table with WHERE hash = any(array-of-encoded-string)        
            ArrayList<String> hexHashList = new ArrayList<>();
            for (byte[] hash : hashes) {
                hexHashList.add("\\x" + Hex.encodeHexString(hash));
            }
            Array stringArray = con.createArrayOf("bytea", hexHashList.toArray());
            System.err.println(String.format("String representation of array: %s", stringArray.toString()));
            pst.setArray(1, stringArray);
            // this either yields 0 rows 
            rs = pst.executeQuery();
            rs.next();
            System.err.println(String.format("found %s rows with array of encoded string", rs.getInt(1)));
            rs.close();

            pst.close();
            con.rollback();
        } catch (UnsupportedEncodingException | SQLException e) {
            throw (Exception) e;
        } finally {
            if (rs != null) {
                try {
                    rs.close();
                } catch (SQLException ignore) {
                }
            }
            if (pst != null) {
                try {
                    pst.close();
                } catch (SQLException ignore) {
                }
            }
            if (con != null) {
                try {
                    con.close();
                } catch (SQLException ignore) {
                }
            }
        }
    }
}
